/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2009 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/
/*
 * A lot of this code originally came from bbClean but most of it has been
 * totally rewritten by me, Tobbe Lundberg.
 * Some of it is from the original xPaintClass code written by Andymon.
 */

#include "stdafx.h"
#include "Gradient.h"
#include <math.h>
#include <float.h>

#define SQF 181  // 181 * 181 * 2   = 65527
#define SQR 256  // sqrt(65527)     = 255.98
#define SQD  16
#define SQR_TAB_SIZE (SQF*SQF*2 / SQD) // ~ 4096

static int isgn(int x)
{
    return x>0 ? 1 : x<0 ? -1 : 0;
}

Gradient::Gradient(int width, int height, COLORREF colour_from, COLORREF colour_to)
{
    colorFrom = colour_from;
    colorTo = colour_to;

    this->width = max(width, 2);
    this->height = max(height, 2);

    int byteSize = width * height;

    int tableSize = width + height;

    if (tableSize < SQR)
    {
        tableSize = SQR;
    }

    // First there's the image pixels, four bytes each (Blue Green Red Alpha)
    // Then there's the x colors
    // After the x colors there are the y colors.

    pixels = new DWORD [byteSize + tableSize];
    xtab = pixels + byteSize;
    ytab = xtab + width;
}

Gradient::~Gradient()
{
    if (pixels)
    {
        delete [] pixels;
        pixels = NULL;
    }
}

DWORD *Gradient::getPixels()
{
    return pixels;
}

void Gradient::verticalGradient()
{
    int y;
    DWORD *pixelPointer = pixels;
    DWORD *yColorScale = ytab;

    createColorScale(ytab, height);

    for (y = 0; y < height; y++)
    {
        // Move to the next row
        pixelPointer = pixels + y*width;

        // Copy the current pixel in the scale
        // to the entire current row
        for (int x = 0; x < width; x++)
        {
            pixelPointer[x] = yColorScale[0];
        }

        // Move to the next pixel in the scale
        ++yColorScale;
    }
}

void Gradient::horizontalGradient()
{
    createColorScale(xtab, width);

    for (int y = 0; y < height; y++)
    {
        memcpy(&pixels[width*y], xtab, width*sizeof(DWORD));
    }
}

void Gradient::ulTolrGradient()
{
    diagonalGradient(double(height)/double(width));
}

void Gradient::urTollGradient()
{
    diagonalGradient(double(height)/double(-width));
}

void Gradient::diagonalGradient(double k)
{
    int y, x1, y1;
    double d, n1;
    double n = 0.0;
    double k1 = -1/k;
    double radius = sqrt((double) height/2*height/2 + width/2*width/2);

    unsigned char *c = (unsigned char *)pixels;

    for (int i = 0; i < height; i++)
    {
        for (int j = 0; j < width; j++)
        {
            y = int(k*(j-width/2) + n);
            n1 = int((i-height/2) - k1*(j-width/2));
            x1 = int((n1-n)/(k-k1));
            y1 = int(k1*x1 + n1);
            d = sqrt(pow((double) (j-width/2)-x1, 2) + pow((double) (i-height/2)-y1, 2)) / (2*radius);
            if ( (i-height/2) > y )
            {
                c[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(colorTo) + (0.5+d)*GetBValue(colorFrom));
                c[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(colorTo) + (0.5+d)*GetGValue(colorFrom));
                c[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(colorTo) + (0.5+d)*GetRValue(colorFrom));
            }
            else
            {
                c[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(colorFrom) + (0.5+d)*GetBValue(colorTo));
                c[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(colorFrom) + (0.5+d)*GetGValue(colorTo));
                c[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(colorFrom) + (0.5+d)*GetRValue(colorTo));
            }
        }
    }
}

void Gradient::quadrantGradient()
{
    DWORD *tlQ; // Top left quadrant
    DWORD *trQ; // Top right quadrant
    DWORD *brQ; // Bottom right quadrant
    DWORD *blQ; // Bottom left quadrant

    tlQ = pixels;

    //
    // We're only actually drawing the top left quadrant.
    // The pixels in this quadrant are then copied to the other three quadrants
    //

    for (int y = 0; y < height / 2; y++)
    {
        trQ = tlQ + width - 1;
        brQ = tlQ + width*(height - 2*y) - 1;
        blQ = brQ - (width - 1);

        for (int x = 0; x < width / 2; x++)
        {
            // x*2 and y*2 is used here to get the full
            // color range. This is needed since we're
            // only going to half the width and height
            // in the for loops
            paintPixel(tlQ, x*2, y*2);

            trQ[0] = tlQ[0];
            blQ[0] = tlQ[0];
            brQ[0] = tlQ[0];

            // Move to the next pixel (towards the center of the horizontal axle)
            ++tlQ;
            ++blQ;
            --trQ;
            --brQ;
        }

        // Move to the next row.
        // We're already at the middle of the image, so we only
        // have to move another half image
        tlQ += (width/2);
    }
}

void Gradient::createColorScale(DWORD *p, int length)
{
    BYTE *c = (BYTE *)p;

    BYTE fromBlue = GetBValue(colorFrom);
    BYTE fromGreen = GetGValue(colorFrom);
    BYTE fromRed = GetRValue(colorFrom);

    int tmp;

    tmp = GetBValue(colorTo) - fromBlue;
    int diffBlue = tmp + isgn(tmp);
    tmp = GetGValue(colorTo) - fromGreen;
    int diffGreen = tmp + isgn(tmp);
    tmp = GetRValue(colorTo) - fromRed;
    int diffRed = tmp + isgn(tmp);

    int i = length - 1;
    while (i >= 0)
    {
        c[0] = fromBlue  + diffBlue  * i / length;
        c[1] = fromGreen + diffGreen * i / length;
        c[2] = fromRed   + diffRed   * i / length;
        c[3] = 0;
        c += 4;
        i--;
    }
}

void Gradient::copyToHdc(HDC hDC, int px, int py, int w, int h)
{
    if (pixels == NULL)
    {
        return;
    }

    BITMAPINFO bv4info;
    setup_bv4info(&bv4info);
    SetDIBitsToDevice(hDC, px, py, w, h, 0, 0, 0, h, pixels, &bv4info, DIB_RGB_COLORS);
}

void Gradient::setup_bv4info(BITMAPINFO *bv4info)
{
    ZeroMemory(bv4info, sizeof(*bv4info));
    bv4info->bmiHeader.biSize = sizeof(bv4info->bmiHeader);
    bv4info->bmiHeader.biWidth = width;
    bv4info->bmiHeader.biHeight = height;
    bv4info->bmiHeader.biPlanes = 1;
    bv4info->bmiHeader.biBitCount = 32;
    bv4info->bmiHeader.biCompression = BI_RGB;
}

