/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2009 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "GradientElliptic.h"

static unsigned long bsqrt(unsigned long x)
{
    unsigned long q, r;
    if (x < 2)
        return x;
    for (r = x >> 1; q = x / r, q < r; r = (r + q) >> 1);
    return r;
}

GradientElliptic::GradientElliptic(int width, int height, COLORREF startColor, COLORREF endColor) : Gradient(width, height, startColor, endColor)
{
    init_sqrt();
    createColorScale(xtab, SQR);
    quadrantGradient();
}

void GradientElliptic::paintPixel(DWORD *p, int x, int y)
{
    int dx = SQF - 1 - SQF * x / width;
    int dy = SQF - 1 - SQF * y / height;
    int f = _sqrt_table[(dx*dx + dy*dy) / SQD];
    *p = xtab[f];
}

void GradientElliptic::init_sqrt()
{
    for (int i = 0; i < SQR_TAB_SIZE; i++)
    {
        // backwards compatible to the elliptic inconsistency in
        // the original gradient code: sqrt(i/2) instead of sqrt(i)
        int f = bsqrt(i*SQD/2);
        _sqrt_table[i] = (unsigned char)(SQR - 1 - f);
    }
}