/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2009 The LS-Universe Team

  Homepage: http://www.ls-universe.info

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "SettingsHelpers.h"

#include "shellapi.h"

//////////////////////////////////////////////////////////////////////
// Exported for Module Loading
//////////////////////////////////////////////////////////////////////

int initModuleEx(HWND hParent, HINSTANCE hInstance, const char *lsPath)
{
    LSSetVariable("xPaintClassVersion", XPCVERSION);

    char szNewXPCPath[MAX_LINE_LENGTH] = {0};
    VarExpansion( szNewXPCPath, "$litestepdir$" );
    strcat(szNewXPCPath, "xpaintclass-1.0.dll");
    
    WIN32_FIND_DATA fd;
    HANDLE hxPCSearch = FindFirstFile(szNewXPCPath, &fd);

    if (INVALID_HANDLE_VALUE == hxPCSearch)
    {   
        //Does Not Exist
        //Install/Copy xPaintClass in LSDir?
        if ( iGetxRCBool("NetLoadModule", "AlwaysUseFolders") )
        {   
            char szOldXPCPath[MAX_LINE_LENGTH] = {0};
            VarExpansion( szOldXPCPath, "$pathtoxpaintclass$" );

            if ( MessageBox(NULL, "You use the feature \"NetLoadModuleAlwaysUseFolders\" of NetLoadModule!\nxPaintClass-1.0 cannot work in combination with this setting!\nYou MUST copy xPaintClass-1.0.dll in your LiteStep folder (next to lsapi.dll)!\n\nShall \"xPaintClass-1.0.dll\" be copied in your LiteStep folder?\n\nCHOOSE YES!! REALLY!\n\nThe theme will recycle after you have choosen YES and display another messagebox with another small task for you!", "Important Information", MB_YESNO | MB_ICONINFORMATION | MB_SETFOREGROUND) == IDYES )
            {
                SHFILEOPSTRUCT shfop;
                shfop.hwnd = NULL;
                shfop.wFunc = FO_COPY;
                shfop.pFrom = szOldXPCPath;
                shfop.pTo = szNewXPCPath;
                shfop.fFlags = FOF_NOCONFIRMMKDIR | FOF_ALLOWUNDO;
                SHFileOperation(&shfop);
                
                Sleep(2000);

                FindClose(hxPCSearch);

                ParseBangCommand(NULL, "!recycle", "");
            }
            else
            {
                if ( MessageBox(NULL, "ARE YOU REALLY SURE?\nTHE THEME OR PARTS OF THE THEME WON'T WORK!\n\nShall \"xPaintClass-1.0.dll\" be copied in your LiteStep folder?\n\nCHOOSE YES!! REALLY!\n\nThe theme will recycle after you have choosen YES and display another messagebox with another small task for you!", "Important Information", MB_YESNO | MB_ICONINFORMATION | MB_SETFOREGROUND) == IDYES )
                {
                    SHFILEOPSTRUCT shfop;
                    shfop.hwnd = NULL;
                    shfop.wFunc = FO_COPY;
                    shfop.pFrom = szOldXPCPath;
                    shfop.pTo = szNewXPCPath;
                    shfop.fFlags = FOF_NOCONFIRMMKDIR | FOF_ALLOWUNDO;
                    SHFileOperation(&shfop);
                    
                    Sleep(2000);

                    FindClose(hxPCSearch);

                    ParseBangCommand(NULL, "!recycle", "");
                }
            }
        }

        FindClose(hxPCSearch);
        
        return 0;
    }
    else 
    {
        if ( iGetxRCBool("NetLoadModule", "AlwaysUseFolders") )
            MessageBox(NULL, "You use the feature \"NetLoadModuleAlwaysUseFolders\" of NetLoadModule!\n\nYou have already installed a version of xPaintClass-1.0 in your LiteStep folder!\nPlease remove the *NetLoadModule line, which loads xPaintClass-1.0 in your current theme!\n\nYou can always remove this line in your downloaded themes from now on.\n\nOnly add the following line:\n\n*NetLoadModule xpaintclass-1.0\n\nas FIRST *NetLoadModule line in your theme.rc, if you release a theme to the public!", "Important Information", MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
        
        //We Don't need the ModuleLoading Stuff
        FindClose(hxPCSearch);

        return 1;
    }
}

void quitModule(HINSTANCE hInstance)
{
}

//////////////////////////////////////////////////////////////////////
// Exported in xExports.h
//////////////////////////////////////////////////////////////////////

BOOL IsOS(const int &os)
{   
    static int osversion = 0;

    static bool bInitialized = false;

    if (!bInitialized)
    {
        OSVERSIONINFO verInfo;
        verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
        GetVersionEx(&verInfo);
        
        switch(verInfo.dwPlatformId)
        {
            case VER_PLATFORM_WIN32_WINDOWS:

                osversion |= OS_9X;
            
                if(verInfo.dwMinorVersion >= 90)
                {
                    osversion |= OS_ME;
                }
                else if(verInfo.dwMinorVersion >= 10)
                {
                    osversion |= OS_98;
                }
                else
                {
                    osversion |= OS_95;
                }
                break;
            
            case VER_PLATFORM_WIN32_NT:
            
                osversion |= OS_NT;
            
                if(verInfo.dwMajorVersion == 5)
                {
                    osversion |= OS_2KXP;

                    if (verInfo.dwMinorVersion >= 1)
                    {
                        osversion |= OS_XP;
                    }
                    else
                    {
                        osversion |= OS_2K;
                    }
                }
				else if (verInfo.dwMajorVersion >= 6)
				{
					osversion |= OS_2KXP;
					osversion |= OS_XP;
					osversion |= OS_VIW7;

					if (verInfo.dwMinorVersion >= 1)
                    {
                        osversion |= OS_WIN7;
                    }
                    else
                    {
                        osversion |= OS_VISTA;
                    }
				}
                else
                {
                    osversion |= OS_NT4;
                }
                break;

            default:;
        }

        bInitialized = true;
    }

    return (osversion & os) ? true : false;
}

int QueryPixelDepth()
{
    DEVMODE DevMode;
    DevMode.dmSize=sizeof(DEVMODE);
    
    if (EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&DevMode))
        return (int) DevMode.dmBitsPerPel;
    else
        return 8;
}

// retrieve handle of the desktop window
HWND GetLitestepDesktop()
{
    HWND hWnd = FindWindow("DesktopBackgroundClass", NULL);

    if(hWnd)
        return hWnd;

    return GetDesktopWindow();
}

// is the given window a toplevel application window?
BOOL IsAppWindow(const HWND &hWnd)
{
    if (!IsWindow(hWnd))
        return false;
    
    if (!IsWindowVisible(hWnd))
        return false;

    if ( GetWindowLong(hWnd, GWL_USERDATA) == MAGIC_DWORD )
        return false;

    long exStyle = GetWindowLong(hWnd, GWL_EXSTYLE);

    if ( exStyle & WS_EX_APPWINDOW )
        return true;
    
    if ( GetParent( hWnd ) )
        return false;
    
    if ( GetWindow(hWnd, GW_OWNER) )
        return false;

    if ( exStyle & WS_EX_TOOLWINDOW )
        return false;

    return true;
}

// modify window styles
void ModifyStyle(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
    SetWindowLong(hWnd, GWL_STYLE, (GetWindowLong(hWnd, GWL_STYLE) & ~removeStyle) | addStyle);
}

void ModifyStyleEx(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
    SetWindowLong(hWnd, GWL_EXSTYLE, (GetWindowLong(hWnd, GWL_EXSTYLE) & ~removeStyle) | addStyle);
}
